@@ -1,22 +1,21 @@
-import os
-
 import click
+import logging
 
 from pypgatk.cgenomes.cgenomes_proteindb import CancerGenomesService
 from pypgatk.commands.utils import print_help
+import pkgutil
 
-this_dir, this_filename = os.path.split(__file__)
+from pypgatk.toolbox.general import read_yaml_from_text, read_yaml_from_file
+default_config_text = pkgutil.get_data(__name__, "../config/cbioportal_config.yaml").decode()
 
+log = logging.getLogger(__name__)
 
 @click.command('cbioportal-to-proteindb', short_help='Command to translate cbioportal mutation data into proteindb')
-@click.option('-c', '--config_file',
-              help='Configuration for cbioportal to proteindb tool',
-              default=this_dir + '/../config/cbioportal_config.yaml')
+@click.option('-c', '--config_file', help='Configuration for cbioportal to proteindb tool')
 @click.option('-in', '--input_mutation', help='Cbioportal mutation file')
 @click.option('-fa', '--input_cds', help='CDS genes from ENSEMBL database')
 @click.option('-out', '--output_db', help='Protein database including all the mutations')
-@click.option('-f', '--filter_column', default='CANCER_TYPE',
-              help='Column in the VCF file to be used for filtering or splitting mutations')
+@click.option('-f', '--filter_column', default='CANCER_TYPE', help='Column in the VCF file to be used for filtering or splitting mutations')
 @click.option('-a', '--accepted_values', default='all',
               help='Limit mutations to values (tissue type, sample name, etc) considered for generating proteinDBs, by default mutations from all records are considered')
 @click.option('-s', '--split_by_filter_column',
@@ -27,6 +26,14 @@ this_dir, this_filename = os.path.split(__file__)
 @click.pass_context
 def cbioportal_to_proteindb(ctx, config_file, input_mutation, input_cds, output_db,
                             clinical_sample_file, filter_column, accepted_values, split_by_filter_column):
+
+  if config_file is None:
+    config_data = read_yaml_from_text(default_config_text)
+    msg = "The default configuration file for cbioportal to proteindb is used: {}".format("cbioportal_config.yaml")
+    log.info(msg)
+  else:
+    config_data = read_yaml_from_file(config_file)
+
   if input_mutation is None or input_cds is None or output_db is None:
     print_help()
 
@@ -38,5 +45,5 @@ def cbioportal_to_proteindb(ctx, config_file, input_mutation, input_cds, output_
                         CancerGenomesService.ACCEPTED_VALUES: accepted_values,
                         CancerGenomesService.SPLIT_BY_FILTER_COLUMN: split_by_filter_column}
 
-  cosmic_to_proteindb_service = CancerGenomesService(config_file, pipeline_arguments)
+  cosmic_to_proteindb_service = CancerGenomesService(config_data, pipeline_arguments)
   cosmic_to_proteindb_service.cbioportal_to_proteindb()
