@@ -47,6 +47,7 @@ class EnsemblDataDownloadService(ParameterConfiguration):
   CONFIG_KEY_SKIP_NCRNA = 'skip_ncrna'
   CONFIG_KEY_SKIP_DNA = 'skip_DNA'
   CONFIG_KEY_SKIP_VCF = 'skip_vcf'
+  CONFIG_KEY_GRCh37 = 'grch37'
 
   def __init__(self, config_file, pipeline_arguments):
     """
@@ -93,7 +94,7 @@ class EnsemblDataDownloadService(ParameterConfiguration):
     self._ensembl_species = species_info['species']
     return self._ensembl_species
 
-  def download_database_by_species(self, grch37=False):
+  def download_database_by_species(self, grch37):
     """
     This method takes a list of Taxonomies from the commandline parameters
     and download the Protein fasta files and the gtf files.
@@ -139,7 +140,7 @@ class EnsemblDataDownloadService(ParameterConfiguration):
               files.extend(vcf_files)
 
             self.get_logger().debug("Files downloaded -- " + ",".join(
-                [x for x in files if x]))
+              [x for x in files if x]))
             total_files.extend(files)
     elif species_name_parameters is not None and len(species_name_parameters) > 0:
       for species_name in species_name_parameters:
@@ -170,7 +171,7 @@ class EnsemblDataDownloadService(ParameterConfiguration):
               files.extend(vcf_files)
 
             self.get_logger().debug("Files downloaded -- " + ",".join(
-                [x for x in files if x]))
+              [x for x in files if x]))
             total_files.extend(files)
     else:
       for species in self._ensembl_species:
@@ -199,7 +200,7 @@ class EnsemblDataDownloadService(ParameterConfiguration):
           files.extend(vcf_files)
 
         self.get_logger().debug("Files downloaded -- " + ",".join(
-            [x for x in files if x]))
+          [x for x in files if x]))
         total_files.extend(files)
 
     return total_files
@@ -305,6 +306,7 @@ class EnsemblDataDownloadService(ParameterConfiguration):
           self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_KEY_ENSEMBL_FTP][
             self.CONFIG_KEY_BASE_URL],
           species['release'], species['name'], file_name)
+      print('Downloading -- ', file_url)
       files.append(
         download_file(file_url, self.get_local_path_root_ensembl_repo() + '/' + file_name, self.get_logger()))
     except KeyError:
@@ -400,7 +402,7 @@ class EnsemblDataDownloadService(ParameterConfiguration):
       if grch37:
         species['assembly'] = 'GRCh37'
       file_name = '{}.{}.dna_sm.toplevel.fa.gz'.format(species['name'][0].upper() + species['name'][1:],
-                                             species['assembly'])
+                                                       species['assembly'])
       file_url = '{}/release-{}/fasta/{}/dna/{}'.format(
         self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_KEY_ENSEMBL_FTP][
           self.CONFIG_KEY_BASE_URL],
